library leo_flutter_index_bar;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

const double rowHeight = 25.0;

class IndexBar extends StatefulWidget {
  final List<Friends> listData;
  final void Function(int selctIndex) indexBarCallBack;

  const IndexBar({Key key, this.listData, this.indexBarCallBack})
      : super(key: key);
  @override
  _IndexBarState createState() => _IndexBarState();
}

class _IndexBarState extends State<IndexBar> {
  @override
  Widget build(BuildContext context) {
    return Positioned(
      child: Container(
        width: 100,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            IndexContentWidget(
              listData: widget.listData,
              indexBarCallBack: widget.indexBarCallBack,
            ),
          ],
        ),
      ),
      right: 0.0,
      top: 0.0,
      bottom: 0.0,
    );
  }
}

class IndexCell extends StatelessWidget {
  final String indexString;
  final Color textColor;

  const IndexCell({Key key, this.indexString, this.textColor})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.center,
      height: rowHeight,
      child: Text(
        indexString,
        style: TextStyle(fontSize: 14, color: textColor),
      ),
    );
  }
}

class IndexContentWidget extends StatefulWidget {
  final List<Friends> listData;
  final void Function(int selctIndex) indexBarCallBack;

  const IndexContentWidget({
    Key key,
    this.listData,
    this.indexBarCallBack,
  }) : super(key: key);

  @override
  _IndexContentWidgetState createState() => _IndexContentWidgetState();
}

class _IndexContentWidgetState extends State<IndexContentWidget> {
  List<String> indexData = ['☆'];
  Color _backColor = Color.fromRGBO(1, 1, 1, 0);
  Color _textColor = Colors.black;
  double bumbleDy = 0.0;
  String bumbleText = '';
  bool bumBleHidden = true;

  @override
  void initState() {
    // TODO: implement initState
    for (Friends friend in widget.listData) {
      if (indexData.contains(friend.indexLetter)) continue;
      indexData.add(friend.indexLetter);
    }
  }

  Widget _indeCellForRowAtIndex(BuildContext context, int index) {
    return IndexCell(
      indexString: indexData[index],
      textColor: _textColor,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 100,
      height: rowHeight * indexData.length,
      child: Row(
        children: <Widget>[
          bumBleHidden == true
              ? Container(width: 80, color: Color.fromRGBO(1, 1, 1, 0))
              : Container(
                  alignment: Alignment(0.0, bumbleDy),
                  width: 80,
                  child: Stack(
                    alignment: Alignment(-0.2, 0.0),
                    children: <Widget>[
                      Image(
                        image: AssetImage('images/气泡.png'),
                        width: 55,
                      ),
                      Text(
                        bumbleText,
                        style: TextStyle(color: Colors.white, fontSize: 20),
                      )
                    ],
                  ),
                ),
          Container(
            width: 20,
            color: _backColor,
            child: GestureDetector(
              onVerticalDragDown: (DragDownDetails details) {
                Offset globalPosition = details.globalPosition;
                int selectIndex = getIndexFromIndexBar(context, globalPosition);
                widget.indexBarCallBack(selectIndex);
                setState(() {
                  _backColor = Color.fromRGBO(0, 0, 0, 0.5);
                  _textColor = Colors.white;
                });
              },
              onVerticalDragUpdate: (DragUpdateDetails details) {
                Offset globalPosition = details.globalPosition;
                int selectIndex = getIndexFromIndexBar(context, globalPosition);
                widget.indexBarCallBack(selectIndex);
                setState(() {
                  _backColor = Color.fromRGBO(0, 0, 0, 0.5);
                  _textColor = Colors.white;
                });
              },
              onVerticalDragEnd: (DragEndDetails details) {
                setState(() {
                  bumBleHidden = true;
                  _backColor = Color.fromRGBO(1, 1, 1, 0);
                  _textColor = Colors.black;
                });
              },
              child: ListView.builder(
                itemBuilder: _indeCellForRowAtIndex,
                itemCount: indexData.length,
                physics: NeverScrollableScrollPhysics(),
              ),
            ),
          )
        ],
      ),
    );
  }

  int getIndexFromIndexBar(BuildContext context, Offset globalPosition) {
    RenderBox box = context.findRenderObject();
    Offset localPosition = box.globalToLocal(globalPosition);
    int selectIndex =
        (localPosition.dy ~/ rowHeight).clamp(0, indexData.length - 1);

    bumbleDy = 2.2 / (indexData.length - 1) * selectIndex - 1.1;
    bumBleHidden = false;
    bumbleText = indexData[selectIndex];

    return selectIndex;
  }
}

class Friends {
  final String imageUrl;
  final String name;
  final String indexLetter;

  Friends({this.imageUrl, this.name, this.indexLetter});
}
